@@ -40,10 +40,7 @@
 
    * could be useful, requires more thinking:
    - aggregate calls to force_redraw()
-     - for now, we call force_redraw() on any possible change
-   - skip draw if not on active desktop
-     - there's an event for that
-     - don't skip update or banner
+     - for now, we call force_redraw() on every possible change
    - move constants from .h to configuration settings or autoconf options
 
    * style issues, don't affect features:
@@ -425,6 +422,12 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 
 	g_debug(WIDGET_NAME": draw()");
 
+	// TODO: handled in expose event and force_redraw() already
+	if (!self->desktop_visible) {
+		g_debug(WIDGET_NAME": desktop not visible, not drawing");
+		return;
+	}
+
 	if (!self->image_current) {
 		g_debug(WIDGET_NAME": draw() without image");
 		self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
@@ -542,6 +545,12 @@ force_redraw (CCCHHDoorIsDesktopWidget *self)
 
 	g_debug(WIDGET_NAME": force_redraw()");
 
+	if (!self->desktop_visible) {
+		g_debug(WIDGET_NAME": desktop not visible, not forcing redraw");
+		self->redraw_required = TRUE;
+		return;
+	}
+
 	// Invalidate the drawing
 	GtkWidget *widget = GTK_WIDGET (self);
 
@@ -693,12 +702,11 @@ fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 		fetch_update_async_read_cb, self);
 }
 
-
 // handle async read callback, begin close, parse input
 static void
 fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
-	gpointer buffer, GnomeVFSFileSize bytes_requested, GnomeVFSFileSize bytes_read,
-	gpointer data)
+	gpointer buffer G_GNUC_UNUSED, GnomeVFSFileSize bytes_requested G_GNUC_UNUSED,
+	GnomeVFSFileSize bytes_read, gpointer data)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
 
@@ -952,6 +960,11 @@ ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *eve
 
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_expose_event()");
 
+	if (!self->desktop_visible) {
+		g_debug(WIDGET_NAME": desktop not visible, ignoring expose event");
+		return;
+	}
+
 	if (!self->image_current) {
 		g_debug(WIDGET_NAME": expose event without image");
 		self->image_current = cairo_image_surface_create_from_png(IMAGE_DOORIS_FNORD);
@@ -974,6 +987,7 @@ ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *eve
 
 	cairo_destroy (cr);
 
+	self->redraw_required = FALSE;
 	return FALSE;
 }
 
@@ -1356,7 +1370,7 @@ connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
 
 // open configuration dialog
 static void 
-ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED, 
+ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget,
   gpointer user_data G_GNUC_UNUSED)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
@@ -1536,6 +1550,23 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 }
 
 
+// handle desktop visibility event
+static void 
+ccchh_dooris_desktop_widget_visible_notify (GtkWidget *widget,
+  gpointer user_data G_GNUC_UNUSED)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (widget);
+ 
+	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_visible_notify()");
+
+	g_object_get (self, "is-on-current-desktop", &self->desktop_visible, NULL);
+	g_debug (WIDGET_NAME": is-on-current-desktop changed. visible: %u", self->desktop_visible);
+
+	// redraw if requested while invisible
+	if (self->desktop_visible && self->redraw_required)
+		force_redraw(self);
+}
+
 
 
 static void
@@ -1698,6 +1729,8 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	self->last_report_ts = 0L;
 	self->too_old = FALSE;
 
+	self->desktop_visible = TRUE;
+
 	// set image to NULL, first expose event will then call update()
 	self->image_current = NULL;
 	self->current_alpha = IMAGE_ALPHA_NORMAL;
@@ -1745,6 +1778,10 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	g_signal_connect (self, "show-settings", 
 		G_CALLBACK (ccchh_dooris_desktop_widget_on_show_settings), NULL);   
 
+	// Handle desktop visibility events
+	g_signal_connect (self, "notify::is-on-current-desktop",
+		G_CALLBACK (ccchh_dooris_desktop_widget_visible_notify), NULL);
+
 	if (gconf_client_dir_exists(self->gconf_client, GCONF_DIR, NULL)) {
 		load_settings(self);
 	}
