@@ -105,6 +105,8 @@ typedef struct {
 	gboolean connected;
 	gboolean highlighted;
 	gboolean too_old;
+	gboolean desktop_visible;
+	gboolean redraw_required;
 	gchar *disconnecting_iap;
 	gchar *connected_bearer;
 	gdouble current_alpha;
